@@ -24,7 +24,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: BLINflag.h,v 1.93 2025/04/22 02:41:16 babolo Exp $
+ * $Id: BLINflag.h,v 1.94 2025/12/21 18:10:58 babolo Exp $
  */
 
 #ifndef  _BABOLO_BLINflag_H_
@@ -806,10 +806,10 @@ int         fpurge      __P((FILE *stream));
 size_t      strlcpy     __P((char * __restrict dst, const char * __restrict src, size_t dsize));
 int         fdclose     __P((FILE *fp, int *fdp));
 FILE       *funopen     __P(( void *cookie
-                            , int    (*readfn) (void *, char *, int)
-                            , int    (*writefn)(void *, const char *, int)
-                            , fpos_t (*seekfn) (void *, fpos_t, int)
-                            , int    (*closefn)(void *)
+                            , int   (*readfn) (void *,       char *, int)
+                            , int   (*writefn)(void *, const char *, int)
+                            , off_t (*seekfn) (void *, off_t       , int)
+                            , int   (*closefn)(void *                   )
                            ));
 
 #endif  /* BLIN_OSLinux */
