@@ -30,7 +30,7 @@ static const char copyright[] = "\
 @(#)Copyright (C)2010..2025 @BABOLO http://www.babolo.ru/\n\
 @(#)All rights reserved.\n\
 @(#)V.M "VERS;
-static const char rcsid[] = "$Id: blin.c,v 1.93 2025/04/22 03:13:40 babolo Exp $";
+static const char rcsid[] = "$Id: blin.c,v 1.94 2025/12/21 18:10:58 babolo Exp $";
 
 #define BLIN_COMPAT VMAJOR
 #define BLIN_INTERNAL
@@ -112,6 +112,15 @@ whatf[] =
 
 # include <stdio_ext.h>
 
+typedef struct fccookie {
+  void   *cookie                             ;
+  FILE   *fp                                 ;
+  int   (*readfn) (void *,       char *, int);
+  int   (*writefn)(void *, const char *, int);
+  off_t (*seekfn) (void *, off_t       , int);
+  int   (*closefn)(void *                   );
+} fccookie;
+
 const int sys_nerr = 134;
 const char * const
 sys_errlist[] =
@@ -328,15 +337,74 @@ fdclose(FILE *fp, int *fdp) {
     return((0 > f1) ? EOF : 0);
 }
 
+static ssize_t
+___funopen_read(void *cookie, char *buf, size_t l) {
+    ssize_t   r  = -1;
+    fccookie *c  = (fccookie *)cookie;
+
+    if  (!c) {
+        errno = EBADF;
+    } else {
+        r = c->readfn(c->cookie, buf, l);
+    }
+    return(r);
+}
+
+static ssize_t
+___funopen_write(void *cookie, const char *buf, size_t l) {
+    ssize_t   r  = -1;
+    fccookie *c  = (fccookie *)cookie;
+
+    if  (!c) {
+        errno = EBADF;
+    } else {
+        r = c->writefn(c->cookie, buf, l);
+    }
+    return(r);
+}
+
+static int
+___funopen_seek(void *cookie, off_t *offset, int whence) {
+    off_t     off;
+    int       r  = -1;
+    fccookie *c  = (fccookie *)cookie;
+
+    if  (!c) {
+        errno = EBADF;
+        goto out;
+    }
+    if  (0 > (off = c->seekfn(c->cookie, *offset, whence))) goto out;
+    *offset = off;
+    r = 0;
+out:
+    return(r);
+}
+
+static int
+___funopen_close(void *cookie) {
+    int       r  = -1;
+    fccookie *c  = (fccookie *)cookie;
+
+    if  (!c) {
+        errno = EBADF;
+    } else {
+        r = c->closefn(c->cookie);
+        free(c);
+    }
+    return(r);
+}
+
 FILE *
-funopen( void    *cookie
-       , int    (*readfn) (void *, char *, int)
-       , int    (*writefn)(void *, const char *, int)
-       , fpos_t (*seekfn) (void *, fpos_t, int)
-       , int    (*closefn)(void *)
+funopen( void   *cookie
+       , int   (*readfn) (void *,       char *, int)
+       , int   (*writefn)(void *, const char *, int)
+       , off_t (*seekfn) (void *, off_t       , int)
+       , int   (*closefn)(void *                   )
        ) {
-    cookie_io_functions_t  a   ;
     const char            *mode;
+    FILE                  *fn  ;
+    cookie_io_functions_t  a   ;
+    fccookie              *c   ;
 
     if  (!readfn) {
         mode = "a";
@@ -345,11 +413,23 @@ funopen( void    *cookie
     } else {
         mode = "r+";
     }
-    a.read  = (__ssize_t (*)(void *, char *, size_t))readfn ;
-    a.write = (__ssize_t (*)(void *, const char *, size_t))writefn;
-    a.seek  = (__ssize_t (*)(void *, __off64_t, int))seekfn;
-    a.close = closefn;
-    return(fopencookie(cookie, mode, a));
+    if  (!(c = malloc(sizeof(fccookie)))) {
+        fn = NULL;
+        goto out;
+    }
+    c->cookie = cookie;
+    c->fp = NULL;
+    c->readfn  = readfn;
+    c->writefn = writefn;
+    c->seekfn  = seekfn;
+    c->closefn = closefn;
+    a.read  = !readfn  ? (ssize_t (*)(void *,       char *, size_t))NULL : ___funopen_read;
+    a.write = !writefn ? (ssize_t (*)(void *, const char *, size_t))NULL : ___funopen_write;
+    a.seek  = !seekfn  ? (int     (*)(void *, off_t *     , int   ))NULL : ___funopen_seek;
+    a.close = !closefn ? (int     (*)(void *                      ))NULL : ___funopen_close;
+    fn = fopencookie(c, mode, a);
+out:
+    return(fn);
 }
 
 #endif /* BLIN_OSLinux */
